function[chain] = AbsoluteChainWithDir(ot)

[m,n] = size(ot);
out = false;
chain_index = 1;
prev_dir = 1;

[row,col] = hasZeros(ot);                % finding the index of the black pixel, that is the most left and up
while (row ~= -1)
    ot(row,col) = 6;                    % mark this pixel as visited
    [dir,dx,dy] = findNext(ot,row,col,prev_dir);
    if dir ~= -1                         % if dir==-1 we are in a dead end, and look for new chain start.
        chain(chain_index) = dir;
        chain_index = chain_index + 1;
        row = row+dy;
        col = col+dx;
        prev_dir = dir;
    else
        [row,col,prev_dir,chain, chain_index] = deadEnd(ot,row,col,chain, chain_index);
    end;
end;
return;

% returns the coordinate of the most up and left pixel contains zero.
% if there is no such pixel, returns [-1,-1].
function[row ,col] = hasZeros(ot)
[m,n] = size(ot);
for i = 1:m
    for j = 1:n
        if ot(i,j) == 0
            row = i;
            col = j;
            return;
        end;
    end;
end;
row = -1;
col = -1;
return;

% findNext - using the absolute chain code, finding location of the next
% pixel containing zero.
% num = the direction number according to the absolute compass.
% dx = the change in the column coordinate.
% dy = the change in the row coordinate.
% prev_dir = the direction we moved last time.
function [num, dx, dy] = findNext(ot, row, col,prev_dir)
dx=0;
dy=0;
vec =  [-1 1; -1 0; -1 -1; 0 -1; 1 -1; 1 0; 1 1; 0 1];
i = 0;
found = false;
while ~found
    if i==0
        stepy = vec(prev_dir,1);
        stepx = vec(prev_dir,2);
        if ot(row+stepy, col+stepx) == 0
            found = true;
            num = prev_dir;
            dx = stepx;
            dy = stepy;
        end;
    else
        if i==4
            tmp = prev_dir + 4;
            if tmp > 8
                tmp = tmp - 8;
            end;
            stepy = vec(tmp,1);
            stepx = vec(tmp,2);
            if ot(row+stepy, col+stepx) == 0
                found = true;
                num = tmp;
                dx = stepx;
                dy = stepy;
            else
                num = -1;
                return;
            end;
        else
            pos1 = prev_dir + i;
            pos2 = prev_dir - i;
            if pos1 > 8
                pos1 = pos1 - 8;
            end;
            if pos2 < 1
                pos2 = pos2 + 8;
            end;
            stepy1 = vec(pos1,1);
            stepx1 = vec(pos1,2);
            if ot(row+stepy1,col+stepx1) == 0
                found = true;
                num = pos1;
                dx = stepx1;
                dy = stepy1;
            else
                stepy2 = vec(pos2,1);
                stepx2 = vec(pos2,2);
                if ot(row+stepy2,col+stepx2) == 0
                    found = true;
                    num = pos2;
                    dx = stepx2;
                    dy = stepy2;
                end;
            end;
        end;
    end;
    i = i+1;
end;
return;

% searches at the environment of the pixel (row,col)
% to find continuation when we reached dead end.
% returns the new pixel and the right direction.
% if there is no continuation, we mark all close zeros in ot.
function[row,col,prev_dir,chain, chain_index] = deadEnd(ot,row,col,chain, chain_index)
dis = 1;
found = false;
while (dis <=4) & (~found)
    for i=-dis:dis
        for j=-dis:dis
            if ot(row+i,col+j) == 0
                found = true;
                row = row+i;
                col = col+j;
                prev_dir = getDir(i,j);
            end;
        end;
    end;
    dis = dis+1;
end;
if ~found
    chain(chain_index) = 9;
    chain_index = chain_index + 1;
    [m,n] = size(ot);
    ot = markBadZeros(ot,m,n);
    [row,col] = hasZeros(ot);
    prev_dir = 6;
end;
return;

% marking as visited all the pixels with value 0 that are close
% to a visited pixel.
% close means that their is a path of zeros between this pixel and 6 pixel.
% retuens the matrix of ot with the changes.
function[new_ot] =  markBadZeros(ot,m,n)
marked_index = 1;
for i=1:m
    for j=1:n
        if ot(i,j) == 6
            marked(marked_index,1) = i;
            marked(marked_index,2) = j;
            marked_index = marked_index +1;
        end;
    end;
end;
for i=1:marked_index-1;
    ot = recMarkBadZeros(ot, marked(i,1), marked(i,2));
end;
new_ot=ot;
return;

% helper of markBadZeros - calling recursively.
function[new_ot] =  recMarkBadZeros(ot, i, j)
for k=-1:1
    for l=-1:1
        if ot(i+k,j+l) == 0
            ot(i+k,j+l) = 6;
            ot = recMarkBadZeros(ot,i+k,j+l);
        end;
    end;
end;
new_ot = ot;
return;

% returns the direction of the pixel (i,j) relative to the current position.
function[dir]  = getDir(i,j)
if i>0
    if j>0
        dir = 7;
    else
        if j==0
            dir = 6;
        else
            dir = 5;
        end;
    end;
else
    if i==0
        if j<0
            dir = 4;
        else
            dir = 8;
        end;
    else
        if j>0
            dir =1;
        else
            if j==0
                dir = 2;
            else
                dir = 3;
            end;
        end;
    end;
end;
return;
