function[chain] = RelativeChain(ot)

[m,n] = size(ot);
out = false;
chain_index = 1;
compass = [-1 1; -1 0; -1 -1; 0 -1; 1 -1; 1 0; 1 1; 0 1];           % initialize the compass towards up: the vector represents the direction of 2

[row,col] = hasZeros(ot);                % finding the index of the black pixel, that is the most left and up
while (row ~= -1)
    ot(row,col) = 6;                     % mark this pixel as visited
    [dir,dx,dy,dilema] = findNext(ot,row,col,compass);

    if dir ~= -1                         % if dir==-1 we are in a dead end, and look for new chain start.
        compass = rotate(compass, dir);
        chain(chain_index) = dir;
        chain_index = chain_index + 1;
        row = row+dy;
        col = col+dx;
    else
        ot = markBadZeros(ot,m,n);
        [row,col] = hasZeros(ot);
    end;
end;

% returns the coordinate of the most up and left pixel contains zero.
% if there is no such pixel, returns [-1,-1].
function[row ,col] = hasZeros(ot)
[m,n] = size(ot);
for i = 1:m
    for j = 1:n
        if ot(i,j) == 0
            row = i;
            col = j;
            return;
        end;
    end;
end;
row = -1;
col = -1;
return;

% findNext - using the relative chain code, finding location of the next
% pixel containing zero.
% dir = the direction number according to the relative compass.
% dx = the change in the column coordinate.
% dy = the change in the row coordinate.
% dilema = true if there was a dilema to choose pixel. (not used for now).
function[dir,dx,dy,dilema] = findNext(ot,row,col,compass)
pos_row = row + compass(2,1);
pos_col = col + compass(2,2);
if ot(pos_row, pos_col) == 0
    dir = 2;
    dilema = false;
else
    row1 = row + compass(1,1);
    col1 = col + compass(1,2);
    row3 = row + compass(3,1);
    col3 = col + compass(3,2);
    if ot(row1, col1) == 0
        if ot(row3, col3) == 0
            dir = decide(1,3);
            dilema = true;
        else
            dir = 1;
            dilema = false;
        end;
    else
        if ot(row3, col3) == 0
            dir = 3;
            dilema = false;
        else
            row0 = row + compass(8,1);
            col0 = col + compass(8,2);
            row4 = row + compass(4,1);
            col4 = col + compass(4,2);
            if ot(row0, col0) == 0
                if ot(row4, col4) == 0
                    dir = decide(8,4);
                    dilema = true;
                else
                    dir = 8;
                    dilema = false;
                end;
            else
                if ot(row4, col4) == 0
                    dir = 4;
                    dilema = false;
                else
                    row5 = row + compass(5,1);
                    col5 = col + compass(5,2);
                    row7 = row + compass(7,1);
                    col7 = col + compass(7,2);
                    if ot(row5, col5) == 0
                        if ot(row7, col7) == 0
                            dir = decide(5,7);
                            dilema = true;
                        else
                            dir = 5;
                            dilema = false;
                        end;
                    else
                        if ot(row7, col7) == 0
                            dir = 7;
                            dilema = false;
                        else                                    % 6 exists only in the first time.
                            row6 = row + compass(6,1);
                            col6 = col + compass(6,2);
                            if ot(row6, col6) == 0
                                dir = 6;
                                dilema = false;
                            else                                 % dead end!!
                                dir = -1;
                                dilema = false;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;
if dir ~= -1
    dx = compass(dir,2);
    dy = compass(dir,1);
else
    dx = 66;
    dy = 66;
end;
return;

% decide which direction to go, in case there are two options.
function[decision] = decide(one, two)                  
    decision = one;                                                                      % this is a random decision, can be changed.
return;

% rotate the compass to direction dir.
function[new_compass] = rotate(compass, dir)              
numOfRot = dir - 2;
if numOfRot == 0
    new_compass = compass;
    return;
else
    if numOfRot > 0                                                                             % rotate left
        for i= 1:8
            if i+numOfRot > 8
                new_compass(i,1) = compass(i+numOfRot - 8,1);
                new_compass(i,2) = compass(i+numOfRot - 8,2);
            else
                new_compass(i,1) = compass(i+numOfRot,1);
                new_compass(i,2) = compass(i+numOfRot,2);
            end;
        end;
    else
        for i=1:8
            if i+numOfRot < 1                                                               % rotate right
                new_compass(i,1) = compass(i+numOfRot + 8,1);
                new_compass(i,2) = compass(i+numOfRot + 8,2);
            else
                new_compass(i,1) = compass(i+numOfRot,1);
                new_compass(i,2) = compass(i+numOfRot,2);
            end;
        end;
    end;
end;

return;

% marking as visited all the pixels with value 0 that are close
% to a visited pixel.
% close means that their is a path of zeros between this pixel and 6 pixel.
% retuens the matrix of ot with the changes.
function[new_ot] =  markBadZeros(ot,m,n)
marked_index = 1;
for i=1:m
    for j=1:n
         if ot(i,j) == 6
            marked(marked_index,1) = i;
            marked(marked_index,2) = j;
            marked_index = marked_index +1;
        end;
    end;
end;
for i=1:marked_index-1;
    ot = recMarkBadZeros(ot, marked(i,1), marked(i,2));
end;
new_ot=ot;
return;

% helper of markBadZeros - calling recursively.
function[new_ot] =  recMarkBadZeros(ot, i, j)
for k=-1:1
    for l=-1:1
        if ot(i+k,j+l) == 0
            ot(i+k,j+l) = 6;
            ot = recMarkBadZeros(ot,i+k,j+l);
        end;
    end;
end;
new_ot = ot;
return;